/*
 *  convert.c - string conversion functions
 *  Copyright (c)2002 - 2004 Andrej  OM1CW
 *  Copyright (c)2005 Mike Karas OM4AA <mike.om4aa@gmail.com>
 *  Copyright (c)2009 Ladislav Vaiz OK1ZIA <ok1zia@nagano.cz>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#define _GNU_SOURCE

#include "osdep.h"
#include "debug.h"
#include "convert.h"
#include "main.h"

float band2freq[BAND_NUM] = {
	0, 50, 70, 144, 432, 1.300, 2.300, 3.400, 5.700, 10, 24, 47, 76, 122, 134,
	248
};

int band2repairlimit[BAND_NUM] = {3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3 }; 

int band2lp[BAND_NUM] = {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 }; 
int bandcat2lp[MAX_CATEGS][MAX_CATEGS];
//int bandcat2lp[BAND_NUM][MAX_CATEGS];
int strictlp = 0;



struct CATEG categs[MAX_CATEGS];
int num_categs = 1;


/* converts string s to uppercase */
char *strupr(char *s)
{
	DEBUG_FUNC("strupr");

//    char *ss = s;
	int i;

    for (i = 0; *(s + i) != '\0'; i++)
		*(s + i) = toupper(*(s + i));

    return s;
}

/* case insensitive strings comparision */
int stricmp(const char *s1, const char *s2)
{
	DEBUG_FUNC("stricmp");

    char us1[BUFF_SIZE + 1], us2[BUFF_SIZE + 1];

    strncpy(us1, s1, sizeof(us1) - 1);
    strncpy(us2, s2, sizeof(us2) - 1);
	us1[BUFF_SIZE] = '\0';
	us2[BUFF_SIZE] = '\0';

    return strcmp(strupr(us1), strupr(us2));
}

#ifndef strncmpi
int strncmpi(char *t1, char *t2, int n)
{
	DEBUG_FUNC("strncmpi");

	char tx1[BUFF_SIZE + 1], tx2[BUFF_SIZE + 1];

	if (n >= BUFF_SIZE)
		return strncmp(t1, t2, n);

	strncpy(tx1, t1, n);
	tx1[BUFF_SIZE] = '\0';
	strupr(tx1);

	strncpy(tx2, t2, n);
	tx2[BUFF_SIZE] = '\0';
	strupr(tx2);

	return strncmp(tx1, tx2, n);
}
#endif

void float_print(float f, char *text3)
{
	DEBUG_FUNC("float_print");

	char pfx = 0;
	char text[15]; /* 7 + 1 + 3 + 1 */

	/*
	 * if (f>=1000) { pfx='k'; f/=1000; } 
	 */
	if (f < 1) {
		pfx = 'm';
		f *= 1000;
	}

	if (f < 1) {
		pfx = 'u';
		f *= 1000;
	}

	sprintf(text, "%7.3f", f);	/* 30.5.2003 urezavalo to potom nuly na
								 * konci */
	char *text2 = strchr(text, '.');
	if (text2)
		*text2 = 0;

	/*
	 * if (pfx) text[3]=0; else text[4]=0; char *text2=strchr(text, 0)-1;
	 * 
	 * while (text2>text && *text2=='0') *(text2--)=0; if (*text2=='.')
	 * *text2=0; 
	 */
	if (pfx)
		sprintf(text3, "%3s %cW", text, pfx);
	else
		sprintf(text3, "%4s W", text);
}


uint8_t str2band(char *cmd_arg)
{
	DEBUG_FUNC("str2band");

	char *temp;
	float freq, tol;
	uint8_t x;

	temp = strchr(cmd_arg, ',');
	if (temp)
		*temp = '.';
	freq = atof(cmd_arg);
	if (freq >= 1000)
		freq /= 1000;
	if (freq < 10)
		tol = 0.11;
	else if (freq < 100)
		tol = 1.1;
	else
		tol = 9; // bylo 11, ale pak 134 GHz bylo 144 MHz
	for (x = 1; x < BAND_NUM; x++) {
		if ((freq >= 100 && band2freq[x] >= 100)
			|| (freq >= 10 && freq < 100 && band2freq[x] >= 10
				&& band2freq[x] < 100) || (freq >= 1 && freq < 10
										   && band2freq[x] >= 1
										   && band2freq[x] < 10))
			if ((band2freq[x] - tol) <= freq
				&& (band2freq[x] + tol) >= freq) {
				return x;
			}
	}

	return 0;
}


char str2cat(char *psect, uint8_t band)
{
	DEBUG_FUNC("str2cat");

	char psect2[BUFF_SIZE + 1], *c;
	int i;

    if (my_strcasestr(psect, "check") != NULL) return 0;

	for (i = 1; i < num_categs; i++){
	    strcpy(psect2, categs[i].psect);
        for (c=strtok(psect2, ",");
             c!=NULL;
             c=strtok(NULL, ";,")){
            
            if ((!categs[i].band || band == categs[i].band)
                && !strcasecmp(c,psect))
			return i;
        }
    }

	return -1;
}

/*
 * 	convert:
 *  (loc >> 23) & 0x1f == loc[0]   ('J') - 5 bits
 *  (loc >> 18) & 0x1f == loc[1]   ('N') - 5 bits
 *  (loc >> 14) & 0x0f == loc[2]   ('8') - 4 bits
 *  (loc >> 10) & 0x0f == loc[3]   ('8') - 4 bits
 *  (loc >>  5) & 0x1f == loc[4]   ('U') - 5 bits
 *  (loc      ) & 0x1f == loc[5]   ('U') - 5 bits
 */
uint32_t loc2num(char *loc)
{
	DEBUG_FUNC("loc2num");

	if (strlen(loc) < 6)
		return 0;

	strupr(loc);

	if (loc[0] < 'A' || loc[0] > 'X' || loc[1] < 'A' || loc[1] > 'X'
		|| loc[2] < '0' || loc[2] > '9' || loc[3] < '0' || loc[3] > '9'
		|| loc[4] < 'A' || loc[4] > 'X' || loc[5] < 'A' || loc[5] > 'X')
		return 0;

	return (0x80000000 | ((loc[0] - 'A') << 23) | ((loc[1] - 'A') << 18) |
			((loc[2] - '0') << 14) | ((loc[3] - '0') << 10) |
			((loc[4] - 'A') << 5) | (loc[5] - 'A'));
}

char *num2loc(uint32_t num, char *loc)
{
	DEBUG_FUNC("num2loc");

	if (!(num & 0x80000000)) {
		strcpy(loc, "INVALD");
		return NULL;
	}

	loc[0] = 'A' + ((num >> 23) & 0x1F);
	loc[1] = 'A' + ((num >> 18) & 0x1F);
	loc[2] = '0' + ((num >> 14) & 0x0F);
	loc[3] = '0' + ((num >> 10) & 0x0F);
	loc[4] = 'A' + ((num >> 5) & 0x1F);
	loc[5] = 'A' + (num & 0x1F);
	loc[6] = 0;

	return loc;
}

char *parse(char **text)
{
	DEBUG_FUNC("parse");

	char *end, *temp;

	while (**text == ' ' || **text == '\t')
		(*text)++;

	end = strchr(*text, 0);
	temp = strchr(*text, ';');
	if (temp && temp < end)
		end = temp;
	// temp=strchr(*text, ',');
	// if (temp && temp<end) end=temp;

	temp = *text;
	*text = end;
	if (**text)
		(*text)++;

	end--;
	while (end > temp && (*end == ' ' || *end == '\t'))
		end--;
	end++;

	*end = 0;

	return temp;
}

char *band2str(char *buf, int band){
	float bnd = band2freq[band];
    
	if (band >= BAND_GHZ && band < BAND_GHZ_ZERO)
		bnd *= 1000;

	if (band >= BAND_GHZ_ZERO)
		sprintf(buf, "G%i", (int)bnd);
	else
		sprintf(buf, "%04i", (int) bnd);

    return buf;
}

char *my_strcasestr(const char *phaystack, const char *pneedle){
    register const unsigned char *haystack, *needle;
    register char b, c;

    haystack = (const unsigned char *) phaystack;
    needle = (const unsigned char *) pneedle;

    b = tolower(*needle);
    if (b != '\0'){
            haystack--;             /* possible ANSI violation */
            do{
                c = tolower(*++haystack);
                if (c == '\0') goto ret0;
            }while (c != b);
        c = tolower(*++needle);
        if (c == '\0') goto foundneedle;
        ++needle;
        goto jin;

        for (;;){
            register char a;
            register const unsigned char *rhaystack, *rneedle;

            do{
                a = tolower(*++haystack);
                if (a == '\0') goto ret0;
                if (a == b) break;
                a = tolower(*++haystack);
                if (a == '\0') goto ret0;
shloop:;
            }while (a != b);

jin:;     
            a = tolower(*++haystack);
            if (a == '\0') goto ret0;
            if (a != c)    goto shloop;

            rhaystack = haystack-- + 1;
            rneedle = needle;
            a = tolower(*rneedle);

            if (tolower(*rhaystack) == a)
                do{
                    if (a == '\0') goto foundneedle;
                    ++rhaystack;
                    a = tolower(*++needle);
                        if (tolower(*rhaystack) != a) break;
                    if (a == '\0') goto foundneedle;
                        
                    ++rhaystack;
                    a = tolower(*++needle);
                }while (tolower(*rhaystack) == a);

            needle = rneedle;       /* took the register-poor approach */

            if (a == '\0') break;
        }
    }
foundneedle:;
    return (char*) haystack;
ret0:;
    return 0;
}

